import { Chunk } from './сhunk';
import { ResolverOptions, InsertRow } from './interface';
import { ChunkPool } from './pool';
declare type OnResolved = (chunk: Chunk) => void | Promise<void>;
/**
 * ChunkResolver is the central element of the application,
 * which implements all the logic of working with the caching process
 * and provides an API for interacting "outside"
 */
export declare class ChunkResolver {
    #private;
    /**
     * Create ChunkResolver instance
     *
     * @param {ChunkPool} chunkPool any type of pool
     * @param {ResolverOptions} options
     */
    constructor(chunkPool: ChunkPool, options: ResolverOptions);
    /**
     * Delayed insertion
     *
     * @warning This method does not guarantee the validation of the transmitted data,
     * which may subsequently affect the outcome of inserting a chunk into the DBMS.
     * Be confident to validate it inself
     *
     * @param {Table} table table name
     * @param {InsertRow[]} rows list of rows
     * @returns
     */
    cache(table: string, rows: InsertRow[]): Chunk;
    /**
     * Public method to resolve chunks
     * You can use it to provide graceful shutdown with forse insertion of all stored data
     *
     * 1. Prohibition of caching in case if there is some concurrent workers
     * 2. Blocking all chunks to avoid concurrent insertion of rows
     * 3. Extracting all chunks from the store
     * 4. Throwing all chunks outside
     *
     * @warning SIGKILL code can't be handled in Node and will lead you to loss of data
     */
    resolveImmediately(): void;
    /**
     * Wrapping emitter to handle completely resolved chunks
     * There you can bridge this cacher with your codebase and finish your insertion staff
     *
     * @param {Function} onResolved
     */
    onResolved(onResolved: OnResolved): void;
    /**
     * Service method to cleanup memory
     */
    stop(): void;
}
export {};
