"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _ChunkResolver_instances, _ChunkResolver_pool, _ChunkResolver_emitter, _ChunkResolver_options, _ChunkResolver_interval, _ChunkResolver_allowCache, _ChunkResolver_resolveConditionally;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChunkResolver = void 0;
const stream_1 = require("stream");
const errors_1 = require("./errors");
const utils_1 = require("./utils");
/**
 * ChunkResolver is the central element of the application,
 * which implements all the logic of working with the caching process
 * and provides an API for interacting "outside"
 */
class ChunkResolver {
    /**
     * Create ChunkResolver instance
     *
     * @param {ChunkPool} chunkPool any type of pool
     * @param {ResolverOptions} options
     */
    constructor(chunkPool, options) {
        _ChunkResolver_instances.add(this);
        _ChunkResolver_pool.set(this, void 0);
        _ChunkResolver_emitter.set(this, void 0);
        _ChunkResolver_options.set(this, void 0);
        _ChunkResolver_interval.set(this, void 0);
        _ChunkResolver_allowCache.set(this, void 0);
        __classPrivateFieldSet(this, _ChunkResolver_pool, chunkPool, "f");
        __classPrivateFieldSet(this, _ChunkResolver_options, options, "f");
        __classPrivateFieldGet(this, _ChunkResolver_pool, "f").setTtlMs(options.ttlMs);
        __classPrivateFieldGet(this, _ChunkResolver_pool, "f").setMaxSize(options.maxSize);
        __classPrivateFieldSet(this, _ChunkResolver_allowCache, true, "f");
        __classPrivateFieldSet(this, _ChunkResolver_emitter, new stream_1.EventEmitter(), "f");
        __classPrivateFieldSet(this, _ChunkResolver_interval, setInterval(() => __classPrivateFieldGet(this, _ChunkResolver_instances, "m", _ChunkResolver_resolveConditionally).call(this), __classPrivateFieldGet(this, _ChunkResolver_options, "f").checkIntervalMs), "f");
    }
    /**
     * Delayed insertion
     *
     * @warning This method does not guarantee the validation of the transmitted data,
     * which may subsequently affect the outcome of inserting a chunk into the DBMS.
     * Be confident to validate it inself
     *
     * @param {Table} table table name
     * @param {InsertRow[]} rows list of rows
     * @returns
     */
    cache(table, rows) {
        if (!__classPrivateFieldGet(this, _ChunkResolver_allowCache, "f")) {
            throw new errors_1.CacheError("E_CACHE_FORBIDDEN" /* E_CODES.E_CACHE_FORBIDDEN */);
        }
        const unblockedChunk = __classPrivateFieldGet(this, _ChunkResolver_pool, "f").getUnblockedChunk(table);
        unblockedChunk.appendRows([...rows]);
        if (unblockedChunk.isExpired() || unblockedChunk.isOverfilled(__classPrivateFieldGet(this, _ChunkResolver_pool, "f").getMaxSize())) {
            unblockedChunk.block();
        }
        return unblockedChunk;
    }
    /**
     * Public method to resolve chunks
     * You can use it to provide graceful shutdown with forse insertion of all stored data
     *
     * 1. Prohibition of caching in case if there is some concurrent workers
     * 2. Blocking all chunks to avoid concurrent insertion of rows
     * 3. Extracting all chunks from the store
     * 4. Throwing all chunks outside
     *
     * @warning SIGKILL code can't be handled in Node and will lead you to loss of data
     */
    resolveImmediately() {
        __classPrivateFieldSet(this, _ChunkResolver_allowCache, false, "f");
        this.stop();
        __classPrivateFieldGet(this, _ChunkResolver_pool, "f").getTables().forEach(table => {
            __classPrivateFieldGet(this, _ChunkResolver_pool, "f").getChunks(table).forEach(chunk => {
                chunk.block();
                __classPrivateFieldGet(this, _ChunkResolver_pool, "f").removeChunk(table, chunk.id);
                __classPrivateFieldGet(this, _ChunkResolver_emitter, "f").emit("chunk:resolved" /* Events.ChunkResolved */, chunk);
            });
        });
    }
    /**
     * Wrapping emitter to handle completely resolved chunks
     * There you can bridge this cacher with your codebase and finish your insertion staff
     *
     * @param {Function} onResolved
     */
    onResolved(onResolved) {
        __classPrivateFieldGet(this, _ChunkResolver_emitter, "f").on("chunk:resolved" /* Events.ChunkResolved */, (chunk) => onResolved(chunk));
    }
    /**
     * Service method to cleanup memory
     */
    stop() {
        clearInterval(__classPrivateFieldGet(this, _ChunkResolver_interval, "f"));
    }
}
exports.ChunkResolver = ChunkResolver;
_ChunkResolver_pool = new WeakMap(), _ChunkResolver_emitter = new WeakMap(), _ChunkResolver_options = new WeakMap(), _ChunkResolver_interval = new WeakMap(), _ChunkResolver_allowCache = new WeakMap(), _ChunkResolver_instances = new WeakSet(), _ChunkResolver_resolveConditionally = function _ChunkResolver_resolveConditionally() {
    for (const table of __classPrivateFieldGet(this, _ChunkResolver_pool, "f").getTables()) {
        const currentPoolSnapshot = __classPrivateFieldGet(this, _ChunkResolver_pool, "f").getChunks(table);
        const expiredChunks = currentPoolSnapshot.filter(chunk => chunk.isExpired());
        const overfilledChunks = currentPoolSnapshot.filter(chunk => chunk.isOverfilled(__classPrivateFieldGet(this, _ChunkResolver_pool, "f").maxSize));
        expiredChunks.forEach(chunk => chunk.block());
        overfilledChunks.forEach(chunk => chunk.block());
        const resolveChunks = (0, utils_1.uniqBy)([...expiredChunks, ...overfilledChunks], chunk => chunk.id);
        __classPrivateFieldGet(this, _ChunkResolver_pool, "f").removeChunks(table, resolveChunks.map(chunk => chunk.id));
        resolveChunks.forEach(chunk => __classPrivateFieldGet(this, _ChunkResolver_emitter, "f").emit("chunk:resolved" /* Events.ChunkResolved */, chunk));
    }
};
